// Matthew Kraus
// PowerMeter.cpp
//

#include "PowerMeter.h"

#include <cmath>

#ifndef M_PI_4     
	#define M_PI_4     0.785398163397448309616
#endif

PowerMeter::PowerMeter()
{
	_value = 0;
	_p = 0;
}

int PowerMeter::getPower()
{	
	int power = (getHeight() / 2 - _value) * _left;
	if (getComponentID() == "L") {
		power = (getHeight() / 2 - _value) * _right;
	}
	
	return power;
}

void PowerMeter::paint(Graphics& g)
{
	g.fillAll(Colours::black);

	// constant power pan rule
	int power = getPower();

	float colorStep = 255.0 / (static_cast<float>(getHeight()) / 4.0);
	
	int delta = power > 0 ? -1 : 1;

	int barSize = 1;
	int numBars = abs(power) / barSize;

	float red = 0.0;
	float green = 255.0;

	for (int i = 0; i <= numBars; i++) {
		int y = getHeight() / 2 + i * barSize * delta;
		
		if (red < 255.0) red += colorStep;
		else green -= colorStep;
		
		if (red > 255.0) red = 255.0;
		if (green < 0.0) green = 0.0;

		g.setColour( Colour(static_cast<int>(red), static_cast<int>(green), 0) );
		g.drawHorizontalLine(y, 0.0, static_cast<float>(getWidth() ) );
	}
}

void PowerMeter::resized()
{
	jassert(getHeight() > 0);
	_value = getHeight() / 2;
}

void PowerMeter::joystickValueChanged (Joystick* joystick)
{
	jassert(getWidth() > 0 && getHeight() > 0); // xxx our component has not been properly sized

	double joyRange = joystick->getVerticalMaximum() - joystick->getVerticalMinimum();	
	double pixRatio = static_cast<double>(getHeight() ) / abs(joyRange);

	_value = static_cast<int>(pixRatio * joystick->getVerticalValue() );

	//
	// PANNING ////////////////////////////////////////////
	//
	double joyHRange = joystick->getHorizontalMaximum() - joystick->getHorizontalMinimum();
	double pixHRatio = 2.0 / abs(joyHRange);

	double center = (joystick->getHorizontalMaximum() + joystick->getHorizontalMinimum()) / 2.0;
	double p = pixHRatio * joystick->getHorizontalValue() - center * pixHRatio;
	jassert(p >= -1.0 && p <= 1.0);

	_left  = cos(M_PI_4 * (p + 1));
	_right = sin(M_PI_4 * (p + 1));
	
	_p = p;
	
	repaint();
}