// Matthew Kraus
// Queue.h

#pragma once

#ifndef QUEUE_INCLUDED
#define QUEUE_INCLUDED

template <typename T>
class Queue {
public:
	Queue();
	Queue(unsigned int size);
	
	~Queue();

	void Create(unsigned int size);
	void Destroy();

	unsigned int NumElementsInQueue();
	unsigned int Capacity();

	bool isEmpty() const;
	bool isFull() const;

	bool Push(T val);
	bool Push(const T* arrayOfValues, unsigned int length);

	bool Pop(T* retVal);
	void Pop(unsigned int numToRemove);

	unsigned int Copy(T* destArray, unsigned int numElements) const;

	//T& operator[](unsigned int idx); 
	const T& operator[](unsigned int idx) const; 
	//bool peekAt(unsigned int idx, T* retVal);

	unsigned int CountSpecialChars() const;

	//bool startsWith(T* arrayOfValues);
	//bool contains(T val);
private:
	T* _circularBuffer;
	unsigned int _size;
	unsigned int _head;
	unsigned int _tail;	
	unsigned int _numElementsInQueue;

}; // end class Queue



///////////////////////////////////////////////////////////////////////////////////////////////////
//
template <class T>
Queue<T>::Queue()
{
	_circularBuffer = nullptr;
	Destroy();
}


///////////////////////////////////////////////////////////////////////////////////////////////////
//
template <class T>
Queue<T>::Queue(unsigned int size)
{
	Create(size);
}


///////////////////////////////////////////////////////////////////////////////////////////////////
//
template <class T>
void Queue<T>::Create(unsigned int size)
{
	_circularBuffer = nullptr;
	Destroy();

	// minimum size of 16
	if (size < 16) size = 16;

	_circularBuffer = new T[size];
	_size = size;
}


///////////////////////////////////////////////////////////////////////////////////////////////////
//
template <class T>
Queue<T>::~Queue()
{
	Destroy();
}


///////////////////////////////////////////////////////////////////////////////////////////////////
//
template <class T>
void Queue<T>::Destroy()
{
	if (_circularBuffer != nullptr) {
		delete [] _circularBuffer;
		_circularBuffer = nullptr;
	}

	_size = 0;
	_head = 0;
	_tail = 0;
	_numElementsInQueue = 0;
}


///////////////////////////////////////////////////////////////////////////////////////////////////
//
template <class T>
bool Queue<T>::isEmpty() const
{
	return _numElementsInQueue == 0;
}


///////////////////////////////////////////////////////////////////////////////////////////////////
//
template <class T>
bool Queue<T>::isFull() const
{
	return _numElementsInQueue == _size;
}


///////////////////////////////////////////////////////////////////////////////////////////////////
//
template <class T>
bool Queue<T>::Push(T val)
{
	if (isFull() ) return false;

	_circularBuffer[_tail++] = val;
	++_numElementsInQueue;

	if (_tail > _size - 1) _tail = 0;

	return true;
}


///////////////////////////////////////////////////////////////////////////////////////////////////
// Pushes an array of values onto the queue.
//
template <class T>
bool Queue<T>::Push(const T* arrayOfValues, unsigned int length)
{
	if (isFull() ) return false;

	unsigned int idx = 0;
	while (_numElementsInQueue != _size && idx < length) {
		_circularBuffer[_tail++] = arrayOfValues[idx++];
		++_numElementsInQueue;
		if (_tail > _size - 1) _tail = 0;
	}
	
	return idx == length; // true if we copied all elements
}


///////////////////////////////////////////////////////////////////////////////////////////////////
//
template <class T>
bool Queue<T>::Pop(T* retVal)
{
	if (isEmpty() ) return false;

	*retVal = _circularBuffer[_head++];
	--_numElementsInQueue;

	if (_head > _size - 1) _head = 0;

	return true;
}


///////////////////////////////////////////////////////////////////////////////////////////////////
// Removes a number of elements from the queue.
//
template <class T>
void Queue<T>::Pop(unsigned int numToRemove)
{
	if (numToRemove > _numElementsInQueue) numToRemove = _numElementsInQueue;

	_head += numToRemove;
	_numElementsInQueue -= numToRemove;
	if (_head > _size - 1) _head -= _size;
}


///////////////////////////////////////////////////////////////////////////////////////////////////
// Returns the number of elements copied.
template <class T>
unsigned int Queue<T>::Copy(T* destArray, unsigned int numElements) const
{
	if (isEmpty() ) return 0;

	unsigned int idx = 0;

	unsigned int numToCopy = numElements;
	if (_numElementsInQueue < numElements) numToCopy = _numElementsInQueue;

	while (idx < numToCopy) {
		destArray[idx] = this->operator[](idx);
		++idx;
	}

	return numToCopy;
}


///////////////////////////////////////////////////////////////////////////////////////////////////
// Reads the value at a given offset in the queue, but does not change the queue.
//
template <class T>
const T& Queue<T>::operator[](unsigned int idx) const
{	
	unsigned int offset;

	// TODO: error checking similar to this
	//if (isEmpty() ) return nullptr;
	//if (idx >= _numElementsInQueue || idx < 0) return nullptr;

	offset = _head + idx;
	if (offset >= _size) offset -= _size;

	return _circularBuffer[offset];
}


///////////////////////////////////////////////////////////////////////////////////////////////////
// Returns the number of special, non-printable characters.
// Currently special characters include: \t \r
template <class T>
unsigned int Queue<T>::CountSpecialChars() const
{
	unsigned int sum = 0;
	unsigned int offset = _head;

	for (unsigned int i = 0; i < _numElementsInQueue; i++) {
		if (_circularBuffer[offset] == '\t' || _circularBuffer[offset] == '\r') ++sum;
		++offset;
		if (offset >= _size) offset -= _size;
	}

	return sum;
}

///////////////////////////////////////////////////////////////////////////////////////////////////
//
template <class T>
unsigned int Queue<T>::NumElementsInQueue()
{
	return _numElementsInQueue;
}


///////////////////////////////////////////////////////////////////////////////////////////////////
//
template <class T>
unsigned int Queue<T>::Capacity()
{
	return _size;
}

#endif // QUEUE_INCLUDED
