// Matthew Kraus
// UARTOutput.h
//
#pragma once

#ifndef INCLUDED_UARTOUTPUT
#define INCLUDED_UARTOUTPUT

#include <boost/asio.hpp>

#include "modules/juce_core/juce_core.h"
#include "modules/juce_graphics/juce_graphics.h"
#include "modules/juce_gui_basics/juce_gui_basics.h"
#include "modules/juce_gui_extra/juce_gui_extra.h"
#include "modules/juce_events/juce_events.h"

#ifndef QUEUE_INCLUDED
	#include "Queue.h"
#endif

#ifndef INCLUDED_POWERMETER
	#include "PowerMeter.h"
#endif

using namespace juce;
using namespace::boost::asio;

class UARTOutput : public Component,
                   public Thread,				   
				   private AsyncUpdater,
				   public Timer
{
public:
	UARTOutput();
	~UARTOutput();

	virtual void paint(Graphics& g);

	virtual void handleAsyncUpdate();

	virtual void resized();

	static unsigned char reverseBitOrder(unsigned char c);

	bool Connect();
	void Disconnect();
	bool RunManual();
	void StopManual();
	
	void SetPortName(const String& portName);

	void SendText(TextEditor &);
	void Send(String );
	
	virtual void timerCallback();
	void setEngines(PowerMeter* portEng, PowerMeter* stbdEng);

	// writes a message to the debug window
	//void log(const String& message);
	
protected:
	virtual void run();	

protected:

private:	
	void render();

	void read_callback(bool& data_available, deadline_timer& timeout, const boost::system::error_code& error, std::size_t bytes_transferred);
	void wait_callback(serial_port& ser_port, const boost::system::error_code& error);

	void send_string(const String& str);
	
	String padZeros(String str);

	boost::asio::io_service* io;
	boost::asio::serial_port* port;	

	ReadWriteLock uartQueueLock;
	Queue<char> uartQueue;
	static const unsigned int MAX_CHARS_TO_DISPLAY = 500;

	char readBuffer[4096]; // TODO: rename

	String portName;

	//char uartBuffer[4096];
	//int uartBufferIdx;

	Font monofont;
	Image uartDisplay;
	
	PowerMeter* portEngineRef;
	PowerMeter* stbdEngineRef;
	
}; // end class UARTOutput

#endif // INCLUDED_UARTOUTPUT